/*
 * Routines for handling CAM5200 IRQ0 and IRQ1 interrupts.
 *
 * 2006 (C) Wolfgang Denk, DENX Software Engineering, <wd@denx.de>
 *
 * This file is licensed under the term of  the GNU General Public License
 * version 2. The program licensed "as is" without any warranty of any
 * kind, whether express or implied.
 *
 */

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/delay.h>
#include <linux/version.h>
#include <linux/init.h>
#include <linux/vmalloc.h>
#include <linux/errno.h>

#include <linux/irq.h>
#include <asm/mpc5xxx.h>
#include <asm/delay.h>

static void cam5200_irq0_handler(int irq, void *dev_id, struct pt_regs *regs)
{
	printk ("cam5200_irq0_handler() called, irq = %d\n", irq);
}

static void cam5200_irq1_handler(int irq, void *dev_id, struct pt_regs *regs)
{
	printk ("cam5200_irq1_handler() called, irq = %d\n", irq);
}

int cam5200_request_interrupts(void)
{
	if (request_irq(MPC5xxx_IRQ0, cam5200_irq0_handler, SA_INTERRUPT,
				"cam5200_irq0", NULL)) {
		printk ("cam5200_init_irqs: request_irq failed for irq 0\n");
		return 1;
	}
	if (request_irq(MPC5xxx_IRQ1, cam5200_irq1_handler, SA_INTERRUPT,
				"cam5200_irq1", NULL)) {
		printk ("cam5200_init_irqs: request_irq failed for irq 1\n");
		free_irq(MPC5xxx_IRQ0, NULL);
		return 1;
	}
	return 0;
}

void cam5200_free_interrupts(void)
{
	free_irq(MPC5xxx_IRQ0, NULL);
	free_irq(MPC5xxx_IRQ1, NULL);
}

/******************************
 **** Module Declarations *****
 ******************************/
int __init cam5200_irq_init(void)
{
	return cam5200_request_interrupts();
}

module_init (cam5200_irq_init);

#ifdef MODULE
static void cam5200_irq_cleanup(void)
{
	cam5200_free_interrupts();
}

module_exit (cam5200_irq_cleanup);
#endif	/* MODULE */

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Wolfgang Denk <wd@denx.de>");
MODULE_DESCRIPTION("CAM5200 IRQ0/IRQ1 driver");
