/*
 * arch/ppc/platform/mpc83xx_common.c
 *
 * MPC83xx common routines
 *
 * Copyright 2005 DENX Software Engineering
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 */

#include <linux/config.h>
#include <linux/stddef.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/errno.h>
#include <linux/reboot.h>
#include <linux/pci.h>
#include <linux/irq.h>

#include <asm/time.h>
#include <asm/io.h>
#include <asm/bootinfo.h>
#include <asm/pci-bridge.h>
#include <asm/mpc83xx.h>
#include <asm/irq.h>
#include <asm/delay.h>

#include <platforms/mpc83xx_common.h>

unsigned char __res[sizeof (bd_t)];

/*  The decrementer counts at the system (internal) clock frequency
 *  divided by 4 */
void __init mpc83xx_calibrate_decr(void)
{
	bd_t *binfo = (bd_t *) __res;
	unsigned int freq, divisor;

	/* get the core frequency */
	freq = binfo->bi_busfreq;

	/* The timebase is updated every 4 bus clocks on e300 core */
	divisor = 4;

	printk("time_init: decrementer frequency = %u.%.6u MHz\n",
			freq / divisor / 1000000, (freq / divisor) % 1000000);

	tb_ticks_per_jiffy = freq / divisor / HZ;
	tb_to_us = mulhwu_scale_factor(freq / divisor, 1000000);

	/* Set the time base to zero */
	mtspr(SPRN_TBWL, 0);
	mtspr(SPRN_TBWU, 0);

	/* Clear any pending timer interrupts */
	mtspr(SPRN_TSR, TSR_ENW | TSR_WIS | TSR_DIS | TSR_FIS);

	/* Enable decrementer interrupt */
	mtspr(SPRN_TCR, TCR_DIE);
}

/* Return the amount of memory */
unsigned long __init mpc83xx_find_end_of_memory(void)
{
	bd_t *binfo;

	binfo = (bd_t *) __res;
	return binfo->bi_memsize;
}

/* Reset cpu */
void mpc83xx_restart(char *cmd)
{
	bd_t *binfo = (bd_t *)__res;

	u32 *rcr = ioremap(binfo->bi_immr_base + MPC83xx_RCR, 4);
	u32 *rpr = ioremap(binfo->bi_immr_base + MPC83xx_RPR, 4);

	/* apply reset protect unlock command to
	 * reset control protection register */
	*rpr = MPC83xx_RPR_RSTE;

	/* apply software hard reset to
	 * reset control register*/
	*rcr = MPC83xx_RCR_SWHR;

	/* not reached, but... */
	iounmap(rcr);
	iounmap(rpr);
	for (;;) ;
}

void mpc83xx_halt(void)
{
	local_irq_disable();
	for (;;) ;
}

void mpc83xx_power_off(void)
{
	mpc83xx_halt();
}

long __init mpc83xx_time_init(void)
{
	bd_t *binfo = (bd_t *)__res;
	u32 *spcr = ioremap(binfo->bi_immr_base + MPC83xx_SPCR, 4);

	*spcr |= MPC83xx_SPCR_TBEN;

	iounmap(spcr);

	return 0;
}

#ifdef CONFIG_PCI
void 
mpc834x_pcibios_fixup(void)
{
	struct pci_dev *dev = NULL;
	unsigned int class;
	int i;
	
	if ((dev = pci_find_device(PCI_VENDOR_ID_FREESCALE,
				   PCI_DEVICE_ID_FREESCALE_MPC8349E, NULL))) {
		class = dev->class >> 8;
		if (class == PCI_CLASS_BRIDGE_OTHER) {
			/*
			 * at least rev 1.1 of the MPC8349E chip has the class
			 * wrongly set in the host/PCI bridge config space
			 * register; this leads to the host/PCI bridge being
			 * treated as a regular client device, trying to assign
			 * PCI resources to it etc.
			 *
			 */
			dev->class = (PCI_CLASS_BRIDGE_HOST << 8) & ~0xf;

			/* for some odd reasons the host/PCI bridge behaves
			 * like an agent device and would claim PCI mem/io/irq
			 * resources, so we make these resources inactive (i.e.
			 * the bridge will not claim resources from itself)
			 */
			for (i = 0; i < DEVICE_COUNT_RESOURCE; i++) 
				dev->resource[i].start = 0;
				dev->resource[i].end = 0;
		}
	}	
}
#endif
