/*
 * Authors: Mark A. Greer <mgreer@mvista.com>, Stefan Roese <sr@denx.de>
 *
 * Adapted from ev64260.c by Stefan Roese (sr@denx.de)
 *
 * Copyright 2001-2003 MontaVista Software Inc.
 * Copyright 2005 DENX Software Engineering (wd@denx.de)
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 */

/*
 * The GT64260 has 2 PCI buses each with 1 window from the CPU bus to
 * PCI I/O space and 4 windows from the CPU bus to PCI MEM space.
 * We'll only use one PCI MEM window on each PCI bus.
 *
 * This is the CPU physical memory map (windows must be at least 1MB and start
 * on a boundary that is a multiple of the window size):
 *
 * 	0xfc000000-0xffffffff		- External FLASH on device module
 * 	0xfbf00000-0xfbffffff		- Embedded (on board) FLASH
 * 	0xfbe00000-0xfbefffff		- GT64260 Registers (preferably)
 * 					  but really a config option
 * 	0xfbd00000-0xfbdfffff		- External SRAM on device module
 * 	0xfbc00000-0xfbcfffff		- TODC chip on device module
 * 	0xfbb00000-0xfbbfffff		- External UART on device module
 * 	0xa2000000-0xfbafffff		- <hole>
 * 	0xa1000000-0xa1ffffff		- PCI 1 I/O (defined in gt64260.h)
 * 	0xa0000000-0xa0ffffff		- PCI 0 I/O (defined in gt64260.h)
 * 	0x90000000-0x9fffffff		- PCI 1 MEM (defined in gt64260.h)
 * 	0x80000000-0x8fffffff		- PCI 0 MEM (defined in gt64260.h)
 *
 * 	XXX: FIXME: PPCBoot uses EVB64260_BRIDGE_REG_BASE as 0xf8000000
 * 	  This happens to not conflict with the current memory map, but
 * 	  it should get fixed eventually anyway.
 */

#ifndef __PPC_PLATFORMS_P3G4_H
#define __PPC_PLATFORMS_P3G4_H

#ifndef	MAX
#define	MAX(a,b)	(((a) > (b)) ? (a) : (b))
#endif

#ifdef CONFIG_P3G4
/*
 * CPU Physical Memory Map setup.
 */
#define	P3G4_EXT_FLASH_BASE		0xff000000
#define	P3G4_EMB_FLASH_BASE		0xff800000

#define	P3G4_EXT_FLASH_SIZE_ACTUAL	0x00800000  /* 8MB Extern FLASH     */
#define	P3G4_EMB_FLASH_SIZE_ACTUAL	0x00800000  /* 8MB of Embed FLASH   */

#define	P3G4_EXT_FLASH_SIZE		MAX(GT64260_WINDOW_SIZE_MIN,	\
					    P3G4_EXT_FLASH_SIZE_ACTUAL)
#define	P3G4_EMB_FLASH_SIZE		MAX(GT64260_WINDOW_SIZE_MIN,	\
					    P3G4_EMB_FLASH_SIZE_ACTUAL)

/*
 * Board-specific IRQ info
 */
#define GPP0                            0
#define GPP1                            1
#define GPP2                            2
#define GPP3                            3

#define	P3G4_PCI_INTA		        (GPP0 + 64)
#define	P3G4_PCI_INTB		        (GPP1 + 64)
#define	P3G4_PCI_INTC		        (GPP2 + 64)
#define	P3G4_PCI_INTD		        (GPP3 + 64)

#define GPP_EXTERNAL_INTERRUPTS		((1<<GPP0) | (1<<GPP1) | (1<<GPP2) | (1<<GPP3))


#define SERIAL_PORT_DFNS

#endif /* CONFIG_P3G4 */

#endif /* __PPC_PLATFORMS_P3G4_H */
