/*
 * arch/ppc/platforms/sorcery.h
 *
 * Definitions for Xerox Sorcery-C MPC8220
 * Based on Alaska souces.
 * Copyright 2005 DENX Software Engineering
 * Wolfgang Denk wd@denx.de
 *
 * Modification of Dale Farnsworth's <dale.farnsworth@mvista.com>
 * version made by Kent Borg.
 *
 * Copyright 2003 Motorola Inc.
 * Copyright 2001, 2004 MontaVista Software Inc.
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 *
 */

#ifndef __PPC_PLATFORMS_SORCERY_H
#define __PPC_PLATFORMS_SORCERY_H

#include <linux/config.h>
#include <asm/mpc8220.h>

#define SORCERY_PCI_DRAM_OFFSET	0
#define SORCERY_ISA_IO_BASE		SORCERY_PCI_SPACE_IO
#define SORCERY_ISA_MEM_BASE		0

/* Sorcery CPLD definitions */
/* CPLD registers base */
#define SORCERY_CPLD_REG_BASE		0xf7e40000

typedef struct cpld_regs
{
	u16 date;
	u16 rev_high;
	u16 rev_low;
	u16 name_high;
	u16 name_low;
	u16 mask0;
	u16 status_clear0;
	u16 mask1;
	u16 status_clear1;
	u16 icntrl_status;
} cpld_regs_t;

/* CPLD Interrupt control registers block 0 */
#define SORCERY_CPLD_MASK0_MIE		(1<<15)
#define SORCERY_CPLD_MASK0_USB_PWR_INT	(1<<9)
#define SORCERY_CPLD_MASK0_FE_INT	(1<<8)
#define SORCERY_CPLD_MASK0_PR_PDY_INT	(1<<7)
#define SORCERY_CPLD_MASK0_FI_INT	(1<<6)
#define SORCERY_CPLD_MASK0_DBUG_INT	(1<<5)
#define SORCERY_CPLD_MASK0_NVM_INT	(1<<4)
#define SORCERY_CPLD_MASK0_IOT_INT	(1<<3)
#define SORCERY_CPLD_MASK0_PAGESYNC	(1<<2)
#define SORCERY_CPLD_MASK0_PITCH_SYNC	(1<<1)
#define SORCERY_CPLD_MASK0_DADH_ACT	(1<<0)

#define SORCERY_CPLD_MASK0_INT_NUM	10

/* CPLD Interrupt control registers block 1 */
#define SORCERY_CPLD_MASK1_MIE		(1<<15)
#define SORCERY_CPLD_MASK1_RISER_A_INT	(1<<8)
#define SORCERY_CPLD_MASK1_RISER_B_INT	(1<<7)
#define SORCERY_CPLD_MASK1_RISER_C_INT	(1<<6)
#define SORCERY_CPLD_MASK1_RISER_D_INT	(1<<5)
#define SORCERY_CPLD_MASK1_CAN_INT	(1<<4)
#define SORCERY_CPLD_MASK1_GEM_INT	(1<<3)
#define SORCERY_CPLD_MASK1_SATA_INT	(1<<2)
#define SORCERY_CPLD_MASK1_MERC_INT	(1<<1)
#define SORCERY_CPLD_MASK1_AQUAR_INT	(1<<0)

#define SORCERY_CPLD_MASK1_INT_NUM	9

#define SORCERY_CPLD_INT_BASE		(MPC8220_PERP_IRQ_BASE + MPC8220_PERP_IRQ_NUM)
#define SORCERY_CPLD_INT_NUM		(SORCERY_CPLD_MASKO_INT_NUM + SORCERY_CPLD_MASK1_INT_NUM)

#define SORCERY_CPLD_INT0		MPC8220_IRQ0
#define SORCERY_CPLD_INT1		MPC8220_IRQ1

/* Global interrupt numbers */
#define SORCERY_DADH_ACT_INT		(SORCERY_CPLD_INT_BASE + 0) /* 60 */
#define SORCERY_PITCH_SYNC_INT		(SORCERY_CPLD_INT_BASE + 1)
#define SORCERY_PAGESYNC_INT		(SORCERY_CPLD_INT_BASE + 2)
#define SORCERY_IOT_INT		(SORCERY_CPLD_INT_BASE + 3)
#define SORCERY_NVM_INT		(SORCERY_CPLD_INT_BASE + 4)
#define SORCERY_DBUG_INT		(SORCERY_CPLD_INT_BASE + 5)
#define SORCERY_FI_INT			(SORCERY_CPLD_INT_BASE + 6)
#define SORCERY_PR_PDY_INT		(SORCERY_CPLD_INT_BASE + 7)
#define SORCERY_FE_INT			(SORCERY_CPLD_INT_BASE + 8)
#define SORCERY_USB_PWR_INT		(SORCERY_CPLD_INT_BASE + 9)
#define SORCERY_AQUAR_INT		(SORCERY_CPLD_INT_BASE + 10)
#define SORCERY_MERC_INT		(SORCERY_CPLD_INT_BASE + 11)
#define SORCERY_SATA_INT		(SORCERY_CPLD_INT_BASE + 12)
#define SORCERY_GEM_INT		(SORCERY_CPLD_INT_BASE + 13)
#define SORCERY_CAN_INT		(SORCERY_CPLD_INT_BASE + 14)
#define SORCERY_RISER_D_INT		(SORCERY_CPLD_INT_BASE + 15)
#define SORCERY_RISER_C_INT		(SORCERY_CPLD_INT_BASE + 16)
#define SORCERY_RISER_B_INT		(SORCERY_CPLD_INT_BASE + 17) /* 77 */
#define SORCERY_RISER_A_INT		(SORCERY_CPLD_INT_BASE + 18)

/* Serial port definitions */

#define RS_TABLE_SIZE				1

/* this is the peripheral bus speed */

#define BASE_BAUD				120000000L

#define STD_PSC_OP						\
{								\
	.magic		= 0,					\
	.baud_base	= BASE_BAUD,				\
	.irq		= MPC8220_PSC_IRQ,			\
	.flags		= ASYNC_BOOT_AUTOCONF|ASYNC_SKIP_TEST,	\
	.iomem_base	= (void *)MPC8220_PSC,			\
	.io_type	= SERIAL_IO_MEM,			\
},
#define SERIAL_PORT_DFNS			STD_PSC_OP

/* Forward declarations */
void __init sorcery_init_irq(void);
int sorcery_get_irq(struct pt_regs *regs);

#endif /* __PPC_PLATFORMS_SORCERY_H */

