/*
 * arch/ppc/platforms/top5200.h
 * 
 * Definitions for EMK TOP5200 processor module support
 * (including derivatives)
 * 
 * Modification of Dale Farnsworth's <dale.farnsworth@mvista.com>
 * version made by Kent Borg.
 *
 * Modification of icecube.h by R.Meyer
 *
 * Copyright 2003 Motorola Inc.
 * Copyright 2003 MontaVista Software Inc.
 * Copyright 2004 EMK Elektronik GmbH
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 */
 /*
 * From Processor to PCI:
 *   PCI Mem Space: 0x90000000 - 0xa0000000 -> 0x90000000 - 0xa0000000 (256 MB)
 *   PCI I/O Space: 0xa0000000 - 0xa1000000 -> 0xa0000000 - 0xa1000000 (16 MB)
 *
 * From PCI to Processor:
 *   System Memory: 0x00000000 -> 0x00000000
 */


#ifndef __PLATFORMS_TOP5200_H
#define __PLATFORMS_TOP5200_H

#define MPC5xxx_PROC_PCI_MEM_START	0x90000000U
#define MPC5xxx_PROC_PCI_MEM_END	0x9fffffffU
#define MPC5xxx_PCI_MEM_START		0x90000000U
#define MPC5xxx_PCI_MEM_END		0x9fffffffU
#define MPC5xxx_PCI_MEM_SIZE		(MPC5xxx_PCI_MEM_END + 1 - MPC5xxx_PCI_MEM_START)

#define MPC5xxx_PCI_MEM_VIRT_START	0x90000000U

#define MPC5xxx_PROC_PCI_IO_START	0xa0000000U
#define MPC5xxx_PROC_PCI_IO_END		0xa0ffffffU
#define MPC5xxx_PCI_IO_START		0xa0000000U
#define MPC5xxx_PCI_IO_END		0xa0ffffffU
#define MPC5xxx_PCI_IO_SIZE		(MPC5xxx_PCI_IO_END + 1 - MPC5xxx_PCI_IO_START)

#define MPC5xxx_PCI_IO_VIRT_START	0xa0000000U

#define MPC5xxx_PCI_DRAM_OFFSET		0x00000000U
#define MPC5xxx_PCI_PHY_MEM_OFFSET	0x00000000U

#define MPC5xxx_ISA_IO_BASE		MPC5xxx_PROC_PCI_IO_START
#define MPC5xxx_ISA_MEM_BASE		MPC5xxx_PROC_PCI_MEM_START

#define RS_TABLE_SIZE  4

#ifdef CONFIG_UBOOT
#define	MPC5xxx_INIT_BAUD_BASE		0
#else
#define	MPC5xxx_INIT_BAUD_BASE		CONFIG_PPC_5xxx_IPBFREQ
#endif

#ifdef CONFIG_EVAL5200
  #define CFG_SRAM_PADDR	0xF0010000
  #define CFG_SRAM_SIZE		0x00001ff8
  #define CFG_RTC_PADDR		0xf0011ff8
  #define CFG_RTC_SIZE		0x00000008
#endif

#ifdef CONFIG_HMI5200
  #define CFG_SRAM_PADDR	0xF0010000
  #define CFG_SRAM_SIZE		0x00007ff0
  #define CFG_RTC_PADDR		0xf0017ff0
  #define CFG_RTC_SIZE		0x00000010
#endif

#define STD_PSC_OP(num)					\
{							\
	.magic = 0,					\
	.baud_base = MPC5xxx_INIT_BAUD_BASE,		\
	.irq = MPC5xxx_PSC##num##_IRQ,			\
	.flags = ASYNC_BOOT_AUTOCONF|ASYNC_SKIP_TEST, 	\
	.iomem_base = (void *)MPC5xxx_PSC##num,		\
	.io_type = SERIAL_IO_MEM,			\
},

#define SERIAL_PORT_DFNS	\
	STD_PSC_OP(1)		\
	STD_PSC_OP(2)		\
	STD_PSC_OP(3)		\
	STD_PSC_OP(6)

#endif /* __PLATFORMS_TOP5200_H */
