/*
 * arch/ppc/platforms/tqm5200.h
 *
 * Definitions for TQ-Systems TQM5200 MPC5xxx Standard Development
 * Platform board support
 *
 * Modification of Dale Farnsworth's <dale.farnsworth@mvista.com>
 * version made by Kent Borg.
 *
 * Copyright 2003 Motorola Inc.
 * Copyright 2003 MontaVista Software Inc.
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 */
 /*
 * From Processor to PCI:
 *   PCI Mem Space: 0x90000000 - 0xa0000000 -> 0x90000000 - 0xa0000000 (256 MB)
 *   PCI I/O Space: 0xa0000000 - 0xa1000000 -> 0xa0000000 - 0xa1000000 (16 MB)
 *
 * From PCI to Processor:
 *   System Memory: 0x00000000 -> 0x00000000
 */


#ifndef __PLATFORMS_TQM5200_H
#define __PLATFORMS_TQM5200_H

#define MPC5xxx_PROC_PCI_MEM_START	0x90000000U
#define MPC5xxx_PROC_PCI_MEM_END	0x9fffffffU
#define MPC5xxx_PCI_MEM_START		0x90000000U
#define MPC5xxx_PCI_MEM_END		0x9fffffffU
#define MPC5xxx_PCI_MEM_SIZE		(MPC5xxx_PCI_MEM_END + 1 - MPC5xxx_PCI_MEM_START)

#define MPC5xxx_PCI_MEM_VIRT_START	0x90000000U

#define MPC5xxx_PROC_PCI_IO_START	0xa0000000U
#define MPC5xxx_PROC_PCI_IO_END		0xa0ffffffU
#define MPC5xxx_PCI_IO_START		0xa0000000U
#define MPC5xxx_PCI_IO_END		0xa0ffffffU
#define MPC5xxx_PCI_IO_SIZE		(MPC5xxx_PCI_IO_END + 1 - MPC5xxx_PCI_IO_START)

#define MPC5xxx_PCI_IO_VIRT_START	0xa0000000U

#define MPC5xxx_PCI_DRAM_OFFSET		0x00000000U
#define MPC5xxx_PCI_PHY_MEM_OFFSET	0x00000000U

#define MPC5xxx_ISA_IO_BASE		MPC5xxx_PROC_PCI_IO_START
#define MPC5xxx_ISA_MEM_BASE		MPC5xxx_PROC_PCI_MEM_START

#ifdef CONFIG_CAM5200
#define CAM5200_NIOS_FLASH_VIRT_START	0xef800000
#define CAM5200_NIOS_FLASH_PHYS_START	0xd0000000
#define CAM5200_NIOS_FLASH_SIZE		0x00400000

#define CAM5200_NIOS_RAM_VIRT_START	0xefc00000
#define CAM5200_NIOS_RAM_PHYS_START	0xd1000000
#define CAM5200_NIOS_RAM_SIZE		0x00200000

#define CAM5200_NIOS_SRAM_VIRT_START	0xefe00000
#define CAM5200_NIOS_SRAM_PHYS_START	0xd1200000
#define CAM5200_NIOS_SRAM_SIZE		0x00008000

#define CAM5200_NIOS_REG_VIRT_START	0xeff00000
#define CAM5200_NIOS_REG_PHYS_START	0xb0000000
#define CAM5200_NIOS_REG_SIZE		0x00001000
#endif

#ifdef CONFIG_PS2MULT
#define RS_TABLE_SIZE  4
#else
#if defined(CONFIG_SPI_EVAL) || defined(CONFIG_TB5200)
#define RS_TABLE_SIZE  2
#elif defined(CONFIG_CAM5200)
#define RS_TABLE_SIZE  6
#else
#define RS_TABLE_SIZE  3
#endif
#endif

#ifdef CONFIG_UBOOT
#define	MPC5xxx_INIT_BAUD_BASE		0
#else
#define	MPC5xxx_INIT_BAUD_BASE		CONFIG_PPC_5xxx_IPBFREQ
#endif

#define CFG_SRAM_PADDR                  0xE5000000
#define CFG_SRAM_SIZE                   0x00080000

#ifdef CONFIG_CAM5200
#define SRAM_REGIONS					\
{							\
	.start_phys = CFG_SRAM_PADDR,			\
	.size = CFG_SRAM_SIZE				\
},							\
{							\
	.start_phys = CAM5200_NIOS_SRAM_PHYS_START,	\
	.size = CAM5200_NIOS_SRAM_SIZE			\
}
#endif

#define STD_PSC_OP(num)					\
{							\
	.magic = 0,					\
	.baud_base = MPC5xxx_INIT_BAUD_BASE,		\
	.irq = MPC5xxx_PSC##num##_IRQ,			\
	.flags = ASYNC_BOOT_AUTOCONF|ASYNC_SKIP_TEST, 	\
	.iomem_base = (void *)MPC5xxx_PSC##num,		\
	.io_type = SERIAL_IO_MEM,			\
},

#if defined(CONFIG_PS2MULT)
#define SERIAL_PORT_DFNS	\
	STD_PSC_OP(1)		\
	STD_PSC_OP(2)		\
	STD_PSC_OP(3)		\
{							\
	.magic = 0,					\
	.baud_base = MPC5xxx_INIT_BAUD_BASE,		\
	.irq = MPC5xxx_IRDA_IRQ,			\
	.flags = ASYNC_BOOT_AUTOCONF|ASYNC_SKIP_TEST, 	\
	.iomem_base = (void *)MPC5xxx_PSC6,		\
	.io_type = SERIAL_IO_MEM,			\
}
#elif defined(CONFIG_SPI_EVAL)
#define SERIAL_PORT_DFNS	\
	STD_PSC_OP(6)		\
	STD_PSC_OP(3)
#elif defined(CONFIG_FO300)
#define SERIAL_PORT_DFNS	\
	STD_PSC_OP(1)		\
	STD_PSC_OP(3)		\
	STD_PSC_OP(6)
#elif defined(CONFIG_TB5200)
#define SERIAL_PORT_DFNS	\
	STD_PSC_OP(1)		\
	STD_PSC_OP(6)
#elif defined(CONFIG_CAM5200)
#define SERIAL_PORT_DFNS	\
	STD_PSC_OP(1)		\
	STD_PSC_OP(2)		\
	STD_PSC_OP(3)		\
	STD_PSC_OP(4)		\
	STD_PSC_OP(5)		\
	STD_PSC_OP(6)
#else /* default */
#define SERIAL_PORT_DFNS	\
	STD_PSC_OP(1)		\
	STD_PSC_OP(2)		\
	STD_PSC_OP(3)
#endif

#endif /* __PLATFORMS_TQM5200_H */
