/*
 * arch/ppc/platform/tqm834x.c
 *
 * TQM834x board specific routines
 *
 * Copyright 2005 DENX Software Engineering
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 */

#include <linux/config.h>
#include <linux/stddef.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/errno.h>
#include <linux/reboot.h>
#include <linux/pci.h>
#include <linux/kdev_t.h>
#include <linux/major.h>
#include <linux/blk.h>
#include <linux/console.h>
#include <linux/delay.h>
#include <linux/irq.h>
#include <linux/seq_file.h>
#include <linux/serial.h>
#include <linux/module.h>

#include <asm/system.h>
#include <asm/page.h>
#include <asm/atomic.h>
#include <asm/time.h>
#include <asm/io.h>
#include <asm/machdep.h>
#include <asm/prom.h>
#include <asm/bootinfo.h>
#include <asm/pci-bridge.h>
#include <asm/mpc83xx.h>
#include <asm/irq.h>
#include <asm/ocp.h>
#include <asm/ipic.h>
#include <asm/ppcboot.h>
#include <asm/pgtable.h>
#include <platforms/mpc83xx_common.h>

#ifndef CONFIG_PCI
unsigned long isa_io_base = 0;
unsigned long isa_mem_base = 0;
#endif

#ifdef CONFIG_PCI
extern void tqm834x_setup_hose(void);
extern int tqm834x_exclude_device(u_char, u_char);
extern int tqm834x_map_irq(struct pci_dev *, unsigned char, unsigned char);
#endif

extern unsigned char __res[sizeof (bd_t)];

extern void gen550_progress(char *, unsigned short);
extern void gen550_init(int, struct serial_struct *);
extern void mpc834x_update_ocp(phys_addr_t immrbar);

/* setup the board related information for the enet controllers */
struct ocp_gfar_data mpc834x_tsec1_def = {
	.interruptTransmit = MPC83xx_IRQ_TSEC1_TX,
	.interruptError = MPC83xx_IRQ_TSEC1_ERROR,
	.interruptReceive = MPC83xx_IRQ_TSEC1_RX,
	.interruptPHY = MPC83xx_IRQ_EXT1,
	.flags = (GFAR_HAS_GIGABIT | GFAR_HAS_MULTI_INTR
			| GFAR_HAS_RMON
			| GFAR_HAS_COALESCE),
	.phyid = 2,
	.phyregidx = 0,
};

struct ocp_gfar_data mpc834x_tsec2_def = {
	.interruptTransmit = MPC83xx_IRQ_TSEC2_TX,
	.interruptError = MPC83xx_IRQ_TSEC2_ERROR,
	.interruptReceive = MPC83xx_IRQ_TSEC2_RX,
	.interruptPHY = MPC83xx_IRQ_EXT2,
	.flags = (GFAR_HAS_GIGABIT | GFAR_HAS_MULTI_INTR
			| GFAR_HAS_RMON
			| GFAR_HAS_COALESCE),
	.phyid = 1,
	.phyregidx = 0,
};

struct ocp_mpc_i2c_data mpc834x_i2c1_def = {
	.flags = MPC_I2C_SEPARATE_DFSRR,
};

/* ************************************************************************ */
static void __init tqm834x_early_serial_map(void)
{
	struct serial_struct serial_req;
	bd_t *binfo = (bd_t *) __res;
	phys_addr_t uart_paddr = binfo->bi_immr_base + MPC83xx_UART0_OFFSET;

	/* Setup serial port access */
	memset(&serial_req, 0, sizeof (serial_req));
	serial_req.baud_base = (binfo->bi_busfreq / 16);
	serial_req.line = 0;
	serial_req.port = 0;
	serial_req.irq = MPC83xx_IRQ_UART1;
	serial_req.flags = STD_COM_FLAGS;
	serial_req.io_type = SERIAL_IO_MEM;
	serial_req.iomem_base = ioremap(uart_paddr, MPC83xx_UART0_SIZE);
	serial_req.iomem_reg_shift = 0;

#if defined(CONFIG_SERIAL_TEXT_DEBUG) || defined(CONFIG_KGDB)
	gen550_init(0, &serial_req);
#endif
	if (early_serial_setup(&serial_req) != 0){
		printk("Early serial init of port 0 failed\n");
	}

	/* Assume early_serial_setup() doesn't modify serial_req */
	uart_paddr = binfo->bi_immr_base + MPC83xx_UART1_OFFSET;
	serial_req.line = 1;
	serial_req.port = 1;
	serial_req.iomem_base = ioremap(uart_paddr, MPC83xx_UART1_SIZE);

#if defined(CONFIG_SERIAL_TEXT_DEBUG) || defined(CONFIG_KGDB)
	gen550_init(1, &serial_req);
#endif

	if (early_serial_setup(&serial_req) != 0)
		printk("Early serial init of port 1 failed\n");
}

/* ************************************************************************ */
int tqm834x_show_cpuinfo(struct seq_file *m)
{
	uint pvid, svid, phid1;
	bd_t *binfo = (bd_t *) __res;

	pvid = mfspr(PVR);
	svid = mfspr(SVR);

	seq_printf(m, "Vendor\t\t: Freescale Inc.\n");

	switch (svid & 0xffff0000) {
		case SVR_8349E:
			seq_printf(m, "Machine\t\t: TQM8349E\n");
			break;
		case SVR_8349:
			seq_printf(m, "Machine\t\t: TQM8349\n");
			break;
		case SVR_8347E_TBGA:
		case SVR_8347E_PBGA:
			seq_printf(m, "Machine\t\t: TQM8347E\n");
			break;
		case SVR_8347_TBGA:
		case SVR_8347_PBGA:
			seq_printf(m, "Machine\t\t: TQM8347\n");
			break;
		case SVR_8343E:
			seq_printf(m, "Machine\t\t: TQM8343E\n");
			break;
		case SVR_8343:
			seq_printf(m, "Machine\t\t: TQM8343\n");
			break;
		default:
			seq_printf(m, "Machine\t\t: unknown\n");
			break;
	}

	seq_printf(m, "core closk\t: %u MHz\n", (int)(binfo->bi_intfreq / 1000000));
	seq_printf(m, "bus closk\t: %u MHz\n", (int)(binfo->bi_busfreq / 1000000));
	seq_printf(m, "PVR\t\t: 0x%x\n", pvid);
	seq_printf(m, "SVR\t\t: 0x%x\n", svid);

	/* Display cpu Pll setting */
	phid1 = mfspr(HID1);
	seq_printf(m, "PLL setting\t: 0x%x\n", ((phid1 >> 24) & 0x3f));

	/* Display the amount of memory */
	seq_printf(m, "Memory\t\t: %d MiB\n", (int)(binfo->bi_memsize / (1024 * 1024)));

	return 0;
}

void __init tqm834x_init_IRQ(void)
{
	bd_t *binfo = (bd_t *) __res;

	u8 senses[8] = {
		0,			/* EXT 0 */
		0,			/* EXT 1 */
#ifdef CONFIG_PCI
		IRQ_SENSE_LEVEL,	/* EXT 2 */
		IRQ_SENSE_LEVEL,	/* EXT 3 */
		0,			/* EXT 4 */
		IRQ_SENSE_LEVEL,	/* EXT 5 */
		IRQ_SENSE_LEVEL,	/* EXT 6 */
#else
		0,			/* EXT 2 */
		0,			/* EXT 3 */
		0,			/* EXT 4 */
		0,			/* EXT 5 */
		0,			/* EXT 6 */
#endif
		IRQ_SENSE_LEVEL,	/* EXT 7 */
	};

	ipic_init(binfo->bi_immr_base + 0x00700, 0, MPC83xx_IPIC_IRQ_OFFSET, senses, 8);

	/* Initialize the default interrupt mapping priorities,
	 * in case the boot rom changed something on us.
	 */
	ipic_set_default_priority();
}

static void __init tqm834x_map_io(void)
{
	bd_t *binfo = (bd_t *) __res;

	/* we steal the lowest ioremap addr for virt space */
	io_block_mapping(VIRT_IMMRBAR, binfo->bi_immr_base, IMMRBAR_SIZE, _PAGE_IO);
}

static __inline__ void tqm834x_set_immr_bat(void)
{
	bd_t *binfo = (bd_t *) __res;

	/* we steal the lowest ioremap addr for virt space */
	mb();
	/* DBATxU[19:29] - size = 1MiB (IMMRBAR_SIZE)
	 * DBATxU[30] - supervisor valid bit = 1
	 * DBATxU[31] - user valid bit = 0 */
	mtspr(SPRN_DBAT1U, VIRT_IMMRBAR | 0x1e);

	/* DBATxL[25:28] - WIMG = 0101 (bypass cache, guarded)
	 * DBATxL[30:31] - RW */
	mtspr(SPRN_DBAT1L, binfo->bi_immr_base | 0x2a);
	mb();
}

/* ************************************************************************
 * Setup the architecture */
static void __init tqm834x_setup_arch(void)
{
	struct ocp_def *def;
	struct ocp_gfar_data *einfo;
	bd_t *binfo = (bd_t *) __res;
	unsigned int freq;

	/* get the core frequency */
	freq = binfo->bi_intfreq;

	if (ppc_md.progress)
		ppc_md.progress("tqm834x_setup_arch(): enter", 0);

	/* Set loops_per_jiffy to a half-way reasonable value,
	   for use until calibrate_delay gets called. */
	loops_per_jiffy = freq / HZ;

#ifdef CONFIG_PCI
	/* setup PCI host bridges */
	tqm834x_setup_hose();
#endif

#ifdef CONFIG_VT
	conswitchp = &dummy_con;
#endif
#ifdef CONFIG_SERIAL
	tqm834x_early_serial_map();
#endif

	def = ocp_get_one_device(OCP_VENDOR_MOTOROLA, OCP_FUNC_GFAR, 0);
	if (def) {
		einfo = (struct ocp_gfar_data *) def->additions;
		memcpy(einfo->mac_addr, binfo->bi_enetaddr, 6);
	}
	def = ocp_get_one_device(OCP_VENDOR_MOTOROLA, OCP_FUNC_GFAR, 1);
	if (def) {
		einfo = (struct ocp_gfar_data *) def->additions;
		memcpy(einfo->mac_addr, binfo->bi_enet1addr, 6);
	}

#ifdef CONFIG_BLK_DEV_INITRD
	if (initrd_start)
		ROOT_DEV = MKDEV(RAMDISK_MAJOR, 0);
	else
#endif
#ifdef  CONFIG_ROOT_NFS
		ROOT_DEV = to_kdev_t(0x00FF);	/* /dev/nfs pseudo device */
#else
		ROOT_DEV = to_kdev_t(0x0301);	/* /dev/hda1 IDE disk */
#endif
	mpc834x_update_ocp(binfo->bi_immr_base);

	if (ppc_md.progress)
		ppc_md.progress("tqm834x_setup_arch(): exit", 0);
}

/* ************************************************************************ */
void __init platform_init(unsigned long r3, unsigned long r4, unsigned long r5, unsigned long r6, unsigned long r7)
{

	/* parse_bootinfo must always be called first */
	parse_bootinfo(find_bootinfo());

	/*
	 * If we were passed in a board information, copy it into the
	 * residual data area.
	 */
	if (r3) {
		memcpy((void *) __res, (void *) (r3 + KERNELBASE),
		       sizeof (bd_t));
	}

#if defined(CONFIG_BLK_DEV_INITRD)
	/*
	 * If the init RAM disk has been configured in, and there's a valid
	 * starting address for it, set it up.
	 */
	if (r4) {
		initrd_start = r4 + KERNELBASE;
		initrd_end = r5 + KERNELBASE;
	}
#endif

	/* Copy the kernel command line arguments to a safe place. */
	if (r6) {
		*(char *) (r7 + KERNELBASE) = 0;
		strcpy(cmd_line, (char *) (r6 + KERNELBASE));
	}

	tqm834x_set_immr_bat();

	/* setup the PowerPC module struct */
	ppc_md.setup_arch = tqm834x_setup_arch;
	ppc_md.show_cpuinfo = tqm834x_show_cpuinfo;

	ppc_md.irq_cannonicalize = NULL;
	ppc_md.init_IRQ = tqm834x_init_IRQ;
	ppc_md.get_irq = ipic_get_irq;

	ppc_md.restart = mpc83xx_restart;
	ppc_md.power_off = mpc83xx_power_off;
	ppc_md.halt = mpc83xx_halt;

	ppc_md.find_end_of_memory = mpc83xx_find_end_of_memory;

	ppc_md.time_init = mpc83xx_time_init;
	ppc_md.set_rtc_time = NULL;
	ppc_md.get_rtc_time = NULL;

	ppc_md.calibrate_decr = mpc83xx_calibrate_decr;
	ppc_md.setup_io_mappings  = tqm834x_map_io;

#ifdef CONFIG_SERIAL
	ppc_md.early_serial_map = tqm834x_early_serial_map;
#ifdef CONFIG_SERIAL_TEXT_DEBUG
	ppc_md.progress = gen550_progress;
#endif
#endif
#ifdef CONFIG_PCI
	ppc_md.pci_exclude_device = tqm834x_exclude_device;
	ppc_md.pci_swizzle = common_swizzle;
	ppc_md.pci_map_irq = tqm834x_map_irq;
	ppc_md.pcibios_fixup = mpc834x_pcibios_fixup;
#endif /* CONFIG_PCI */
	if (ppc_md.progress)
		ppc_md.progress("tqm834x_init(): exit", 0);

	return;
}
