/*
 * arch/ppc/platform/tqm834x_pci.c
 *
 * TQM834x PCI routines 
 *
 * Copyright 2005 DENX Software Engineering
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 *
 * Derived from linux-2.6.14-rc3/arch/ppc/syslib/ppc83xx_setup.c
 *
 */

#include <linux/config.h>
#include <linux/stddef.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/errno.h>
#include <linux/pci.h>
#include <linux/delay.h>
#include <asm/machdep.h>
#include <asm/pci-bridge.h>
#include <asm/mpc83xx.h>
#include <asm/immap_83xx.h>
#include <platforms/mpc83xx_common.h>
#include <platforms/tqm834x_pci.h>

#ifdef CONFIG_PCI
/*
 * interrupt routing
 */
inline int
tqm834x_map_irq(struct pci_dev *dev, unsigned char idsel, unsigned char pin)
{
	int irq;
	static char pci_irq_table[][4] =
	    /*
	     *      PCI IDSEL/INTPIN->INTLINE
	     *       A      B      C      D
	     */
	{
		{PIRQA, PIRQB,  PIRQC,  PIRQD}, /* idsel 0x1c */
		{PIRQB, PIRQC,  PIRQD,  PIRQA}, /* idsel 0x1d */
		{PIRQC, PIRQD,  0,  0}  	/* idsel 0x1e */
	};

	const long min_idsel = 0x1c, max_idsel = 0x1e, irqs_per_slot = 4;
	irq = PCI_IRQ_TABLE_LOOKUP;
	if (!irq)
		irq = 0;
	return (irq);
}

int
tqm834x_exclude_device(u_char bus, u_char devfn)
{
	return PCIBIOS_SUCCESSFUL;
}

/*
 * NOTICE: MPC8349 internally has two PCI controllers (PCI1 and PCI2) but since
 * per TQM834x design physical connections to external devices (PCI sockets)
 * are routed only to the PCI1 we do not account for the second one - this code
 * supports PCI1 module only. Should support for the PCI2 be required in the
 * future it needs a separate tqm834x_setup_pci2() function and adequate
 * extensions in the tqm834x_setup_hose() function.
 *
 */
static void __init
tqm834x_setup_pci1(struct pci_controller *hose)
{
	u16 reg16;
	volatile immr_pcictrl_t * pci_ctrl;
	volatile immr_ios_t * ios;
	bd_t *binfo = (bd_t *) __res;

	pci_ctrl = ioremap(binfo->bi_immr_base + 0x8500, sizeof(immr_pcictrl_t));
	ios = ioremap(binfo->bi_immr_base + 0x8400, sizeof(immr_ios_t));

	/*
	 * Configure PCI Outbound Translation Windows
	 */
	ios->potar0 = (MPC83xx_PCI1_LOWER_MEM >> 12) & POTAR_TA_MASK;
	ios->pobar0 = (MPC83xx_PCI1_LOWER_MEM >> 12) & POBAR_BA_MASK;
	ios->pocmr0 = POCMR_EN |
		(((0xffffffff - (MPC83xx_PCI1_UPPER_MEM -
				MPC83xx_PCI1_LOWER_MEM)) >> 12) & POCMR_CM_MASK);

	/* mapped to PCI1 IO space */
	ios->potar1 = (MPC83xx_PCI1_LOWER_IO >> 12) & POTAR_TA_MASK;
	ios->pobar1 = (MPC83xx_PCI1_IO_BASE >> 12) & POBAR_BA_MASK;
	ios->pocmr1 = POCMR_EN | POCMR_IO |
		(((0xffffffff - (MPC83xx_PCI1_UPPER_IO -
				MPC83xx_PCI1_LOWER_IO)) >> 12) & POCMR_CM_MASK);

	/*
	 * Configure PCI Inbound Translation Windows
	 */
	pci_ctrl->pitar1 = 0x0;
	pci_ctrl->pibar1 = 0x0;
	pci_ctrl->piebar1 = 0x0;
	pci_ctrl->piwar1 = PIWAR_EN | PIWAR_PF | PIWAR_RTT_SNOOP | PIWAR_WTT_SNOOP | PIWAR_IWS_2G;

	/*
	 * Release PCI RST signal
	 */
	pci_ctrl->gcr = 0;
	udelay(2000);
	pci_ctrl->gcr = 1;
	udelay(2000);

	reg16 = 0xff;
	early_read_config_word(hose, hose->first_busno, 0, PCI_COMMAND, &reg16);
	reg16 |= PCI_COMMAND_SERR | PCI_COMMAND_MASTER | PCI_COMMAND_MEMORY;
	early_write_config_word(hose, hose->first_busno, 0, PCI_COMMAND, reg16);

	/*
	 * Clear non-reserved bits in status register.
	 */
	early_write_config_word(hose, hose->first_busno, 0, PCI_STATUS, 0xffff);
	early_write_config_byte(hose, hose->first_busno, 0, PCI_LATENCY_TIMER, 0x80);

	iounmap((void *)pci_ctrl);
	iounmap((void *)ios);
}

void __init
tqm834x_setup_hose(void)
{
	struct pci_controller *hose;
	bd_t *binfo = (bd_t *) __res;
	u32 val32;
	volatile immr_clk_t * clk;
	
	clk = ioremap(binfo->bi_immr_base + 0xA00, sizeof(immr_clk_t));

	/*
	 * Configure PCI_CLK_OUTPUT
	 */
	val32 = clk->occr;
	udelay(2000);
	
#ifdef CONFIG_TQM834x
	/*
	 * WARNING! only PCI_CLK_OUTPUT1 is enabled for the TQM834x as this is
	 * the one line actually used for clocking all external PCI devices in
	 * current setup of the STK85xx.  Enabling other PCI_CLK_OUTPUT lines
	 * may lead to board's hang for unknown reasons - particularly
	 * PCI_CLK_OUTPUT6 and PCI_CLK_OUTPUT7 are known to hang the board;
	 * this issue is under investigation (18 oct 05)
	 */
	val32 = OCCR_PCICOE1; 
#else
	/* enable all PCI_CLK_OUTs */
	val32 = (OCCR_PCICOE0 | OCCR_PCICOE1 | OCCR_PCICOE2 | OCCR_PCICOE3 \
		 | OCCR_PCICOE4 | OCCR_PCICOE5 | OCCR_PCICOE6 | OCCR_PCICOE7);
#endif
	/* set frequency of the PCI and clock outputs for external devicesc
	 * according to system clocking settings */
	if (clk->spmr & SPMR_CKID) {
		/* PCI Clock is 1/2 of CONFIG_83XX_CLKIN so need to set up OCCR
		 * fields accordingly */
		val32 |= (OCCR_PCI1CR | OCCR_PCI2CR);
		
		val32 |= (OCCR_PCICD0 | OCCR_PCICD1 | OCCR_PCICD2 \
			  | OCCR_PCICD3 | OCCR_PCICD4 | OCCR_PCICD5 \
			  | OCCR_PCICD6 | OCCR_PCICD7);
	}
	
	clk->occr = val32;
	udelay(2000);
	iounmap((void *)clk);

	hose = pcibios_alloc_controller();

	if (!hose)
		return;

	hose->first_busno = 0;
	hose->bus_offset = 0;
	hose->last_busno = 0xff;

	setup_indirect_pci(hose, binfo->bi_immr_base + PCI1_CFG_ADDR_OFFSET,
			   binfo->bi_immr_base + PCI1_CFG_DATA_OFFSET);
	hose->set_cfg_type = 1;

	tqm834x_setup_pci1(hose);

	hose->pci_mem_offset = MPC83xx_PCI1_MEM_OFFSET;
	hose->mem_space.start = MPC83xx_PCI1_LOWER_MEM;
	hose->mem_space.end = MPC83xx_PCI1_UPPER_MEM;

	hose->io_space.start = MPC83xx_PCI1_LOWER_IO;
	hose->io_space.end = MPC83xx_PCI1_UPPER_IO;
	hose->io_base_phys = MPC83xx_PCI1_IO_BASE;
	isa_io_base =
	    (unsigned long) ioremap(MPC83xx_PCI1_IO_BASE,
				    MPC83xx_PCI1_IO_SIZE);
	hose->io_base_virt = (void *) isa_io_base;

	/* setup resources */
	pci_init_resource(&hose->mem_resources[0],
			  MPC83xx_PCI1_LOWER_MEM,
			  MPC83xx_PCI1_UPPER_MEM,
			  IORESOURCE_MEM, "PCI host bridge");

	pci_init_resource(&hose->io_resource,
			  MPC83xx_PCI1_LOWER_IO,
			  MPC83xx_PCI1_UPPER_IO,
			  IORESOURCE_IO, "PCI host bridge");

	hose->last_busno = pciauto_bus_scan(hose, hose->first_busno);
	return;
}
#endif /* CONFIG_PCI */
