/*
 * This is a test module for the Tundra Universe VMEbus interrupt
 * support.
 *
 * (C) Copyright 2005 Wolfgang Denk, wd@denx.de
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 */

/*
 * Standard in kernel modules
 */
#include <linux/kernel.h>
#include <linux/delay.h>
#include <linux/module.h>
#include <linux/miscdevice.h>
#include <linux/devfs_fs_kernel.h>
#include <linux/rtc.h>
#include <linux/smp_lock.h>
#include <linux/version.h>
#include <linux/init.h>
#include <asm/types.h>
#include <asm/uaccess.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/time.h>
#include <asm/machdep.h>		/* ppc_md */

#include <asm/universe.h>


#define UNIVERSE_TEST_VECTOR    0x72
#define UNIVERSE_TEST_LEVEL     2


/***************************************************************************
 * Debug stuff
 ***************************************************************************
 */
#define	DEBUG

#ifdef	DEBUG
# define debugk(fmt,args...)	printk(fmt ,##args)
# define DEBUG_OPTARG(arg)	arg,
#else
# define debugk(fmt,args...)
# define DEBUG_OPTARG(arg)
#endif

/*
 * Deal with CONFIG_MODVERSIONS
 */
#if CONFIG_MODVERSIONS==1
/* # define MODVERSIONS */
# include <linux/modversions.h>
#endif

/*
 * For character devices
 */
#include <linux/fs.h>		/* character device definitions	*/
#include <linux/wrapper.h>	/* wrapper for compatibility with future versions */


/*
 * Device Declarations
 */

/*
 * The name for our device, as it will appear in /proc/devices
 */
#define DEVICE_NAME		"universe_int_test"
#define	UNIVERSE_INTR_VERSION	"0.1-wd"

/***************************************************************************
 * Local stuff
 ***************************************************************************
 */

/*
 * Prototypes
 */

int init_module(void);
void cleanup_module(void);

static void test_int_handler (int irq, void *arg);


/***************************************************************************
 *
 * Initialize the module
 */
int __init universe_test_init (void)
{
	printk (KERN_INFO
		"Universe extended interrupt test module version "
		UNIVERSE_INTR_VERSION
		"\n"
	);

	/*
	 * Request interrupt and enable VMEbus interrupt level in Universe
	 */
	universe_request_irq(UNIVERSE_TEST_VECTOR, test_int_handler, (void *)0x12345678);
	universe_int_enable(UNIVERSE_TEST_LEVEL);

	debugk ("Universe Interrupt Test installed\n");

	return 0;
}

/***************************************************************************
 * Module Declarations
 ***************************************************************************
 */
#ifdef MODULE
/*
 * Initialize the module
 */
int init_module (void)
{
	debugk ("Initialize Universe Interrupt Test\n");

	universe_test_init ();

	return (0);
}

/*
 * Cleanup - unregister the appropriate file from /proc
 */
void cleanup_module (void)
{
	debugk ("Cleanup Universe Interrupt Test\n");

	universe_free_irq (UNIVERSE_TEST_VECTOR);

	debugk ("Cleanup completed\n");
}

MODULE_AUTHOR("Stefan Roese <sr@denx.de>");
MODULE_DESCRIPTION("Tundra Universe VMEbus interrupt test driver");
MODULE_LICENSE ("GPL");

#endif	/* MODULE */

/***************************************************************************
 * Utility functions
 ***************************************************************************
 */
static void test_int_handler (int irq, void *arg)
{
	printk ("## Test interrupt handler called, IRQ=0x%x, arg=0x%x\n", irq, (int)arg);
}

