/*
    (C) Copyright 2006 Wolfgang Denk, DENX Software Engineering, wd@denx.de.

    This file based on lm75.h that has:
    Copyright (c) 2003 Mark M. Hoffman <mhoffman@lightlink.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*
    This file contains common code for encoding/decoding LM73 type
    temperature readings, which are emulated by many of the chips
    we support.  As the user is unlikely to load more than one driver
    which contains this code, we don't worry about the wasted space.
*/

#include <linux/i2c-proc.h>


/* LM73 registers */
#define LM73_REG_TEMP		0x00
#define LM73_REG_CONF		0x01
#define LM73_REG_T_HIGH		0x02
#define LM73_REG_T_LOW		0x03
#define LM73_REG_ID		0x07

/* Identicication register */
#define LM73_MANUFACTURER_MASK	0xff00
#define LM73_PRODUCT_MASK	0x00f0
#define LM73_REVISION_MASK	0x000f

#define LM73_MANUFACTURER	0x0100
#define LM73_PRODUCT		0x0090

/* driver identification */
#define LM73_DATE		"2006-12-20"
#define LM73_VERSION		"1"
#define LM73_TYPE_NAME		"lm73"
#define LM73_CLIENT_NAME	"LM73 chip"

/* straight from the datasheet */
#define LM73_TEMP_MIN		(-4000)	/* -40 C x 10^magnitude */
#define LM73_TEMP_MAX		15000	/* 150 C x 10^magnitude */


/* TEMP: 0.01C/bit (-256C to +255.75C)
   REG: (0.25C/bit, two's complement) << 5 */
static inline u16 LM73_TEMP_TO_REG(int temp)
{
	int ntemp = SENSORS_LIMIT(temp, LM73_TEMP_MIN, LM73_TEMP_MAX);
	ntemp += (ntemp < 0 ? -12 : 12); /* for proper rounding */
	return (u16)((ntemp / 25) << 5);
}

static inline int LM73_TEMP_FROM_REG(u16 reg)
{
	/* use integer division instead of equivalent right shift to
	   guarantee arithmetic shift and preserve the sign */
	return ((s16)reg / 32) * 25;
}
