/*
 * (C) Copyright 2006
 * EMK Elektronik GmbH
 * Reinhard Meyer, info@emk-elektronik.de
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/errno.h>
#include <linux/i2c.h>
#include <linux/i2c-algo-bit.h>
#include <asm/mpc5xxx.h>

#define SDA0		0x40
#define SCL0		0x80
#define GPIOE0		*((volatile unsigned char*)(MPC5xxx_MBAR+0x0c00))
#define DDR0		*((volatile unsigned char*)(MPC5xxx_MBAR+0x0c08))
#define DVO0		*((volatile unsigned char*)(MPC5xxx_MBAR+0x0c0c))
#define DVI0		*((volatile unsigned char*)(MPC5xxx_MBAR+0x0c20))
#define ODE0		*((volatile unsigned char*)(MPC5xxx_MBAR+0x0c04))

static void bit_top5200_setscl(void *data, int state)
{
	if (state)
		DVO0 |= SCL0;
	else
		DVO0 &= ~SCL0;
}

static void bit_top5200_setsda(void *data, int state)
{
	if (state)
		DVO0 |= SDA0;
	else
		DVO0 &= ~SDA0;
}

static int bit_top5200_getscl(void *data)
{
	return (DVI0 & SCL0) ? 1 : 0;
}

static int bit_top5200_getsda(void *data)
{
	return (DVI0 & SDA0) ? 1 : 0;
}

static void top5200_inc_use(struct i2c_adapter *adap)
{
#ifdef MODULE
	MOD_INC_USE_COUNT;
#endif
}

static void
top5200_dec_use(struct i2c_adapter *adap)
{
#ifdef MODULE
	MOD_DEC_USE_COUNT;
#endif
}

static struct i2c_algo_bit_data top5200_algo_data = {
	NULL,
	bit_top5200_setsda,
	bit_top5200_setscl,
	bit_top5200_getsda,
	bit_top5200_getscl,
	10, 1, 10,		/* udelay, mdelay, timeout */
};

static struct i2c_adapter top5200_board_adapter = {
	name:              "TOP5200 I2C adapter (Bus 0)",
	id:                0xfffc,
	algo:              NULL,
	algo_data:         &top5200_algo_data,
	inc_use:           top5200_inc_use,
	dec_use:           top5200_dec_use,
};

MODULE_DESCRIPTION("I2C adapter for TOP5200 Bus 0");
MODULE_LICENSE("GPL");

static __init int
i2c_top5200_init(void)
{
	/* init the port WakeUp GPIO */
	GPIOE0	|=  (SDA0|SCL0);		/* make them normal I/O pins			*/
	ODE0	|=  (SDA0|SCL0);		/* use open collector feature			*/
	DVO0	|=  (SDA0|SCL0);		/* leave SDA and SCL high				*/
	DDR0	|=  (SDA0|SCL0);		/* set to output						*/

	if (i2c_bit_add_bus(&top5200_board_adapter) < 0)
		return -ENODEV;

	return 0;
}

static __exit void
i2c_top5200_exit(void)
{
	i2c_bit_del_bus(&top5200_board_adapter);
}

module_init(i2c_top5200_init);
module_exit(i2c_top5200_exit);

