/***********************************************************************
 *
 * (C) Copyright 2005
 * DENX Software Engineering
 * Wolfgang Denk, wd@denx.de
 * All rights reserved.
 ***********************************************************************/

/* ------------------------------------------------------------------------- */
/* i2c-tqm85xx.c I2C Interface driver module for the TQM85xx boards          */
/* ------------------------------------------------------------------------- */
/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.		     */
/* ------------------------------------------------------------------------- */

#include <linux/kernel.h>
#include <linux/ioport.h>
#include <linux/module.h>
#include <linux/delay.h>
#include <linux/slab.h>
#include <linux/version.h>
#include <linux/init.h>
#include <asm/irq.h>
#include <asm/io.h>
#include <platforms/tqm85xx_common.h>

#include <linux/i2c.h>
#include <linux/i2c-algo-85xx.h>


static int tqm85xx_i2c_reg(struct i2c_client *client)
{
	return 0;
}


static int tqm85xx_i2c_unreg(struct i2c_client *client)
{
	return 0;
}

static void tqm85xx_i2c_inc_use(struct i2c_adapter *adap)
{
#ifdef MODULE
	MOD_INC_USE_COUNT;
#endif
}

static void tqm85xx_i2c_dec_use(struct i2c_adapter *adap)
{
#ifdef MODULE
	MOD_DEC_USE_COUNT;
#endif
}


static struct i2c_algo_85xx_data tqm85xx_i2c_data = {
	NULL
};

static struct i2c_adapter tqm85xx_i2c_ops = {
	"TQM85xx I2C Interface",
	I2C_HW_MPC85xx_TQM85xx,
	NULL,
	&tqm85xx_i2c_data,
	tqm85xx_i2c_inc_use,
	tqm85xx_i2c_dec_use,
	tqm85xx_i2c_reg,
	tqm85xx_i2c_unreg,
};

int __init i2c_tqm85xx_init(void)
{
	int res = -ENODEV;

	printk(KERN_INFO "i2c-tqm85xx.o: i2c TQM85xx module\n");

	tqm85xx_i2c_data.i2c = ioremap(BOARD_CCSRBAR + M85xx_I2C_OFFSET, M85xx_I2C_SIZE);
	if (i2c_85xx_add_bus(&tqm85xx_i2c_ops) < 0) {
		goto Done;
	}

	printk(KERN_INFO "i2c-tqm85xx.o: found device at 0x%08x.\n",
	       (unsigned int)tqm85xx_i2c_data.i2c);

	res = 0;
Done:
	return res;
}

void __exit i2c_tqm85xx_exit(void)
{
	i2c_85xx_del_bus(&tqm85xx_i2c_ops);
}

EXPORT_NO_SYMBOLS;


#ifdef MODULE
MODULE_AUTHOR("Wolfgang Denk <wd@denx.de>");
MODULE_DESCRIPTION("I2C-Bus driver routines for the TQM85xx boards");

module_init(i2c_tqm85xx_init);
module_exit(i2c_tqm85xx_exit);
#endif
