/*
 * drivers/mtd/maps/uc101.c
 *
 * MTD mapping for the UC1001 board.
 *
 */
#include <linux/module.h>
#include <linux/init.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <asm/io.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/map.h>
#include <linux/mtd/partitions.h>

#ifdef CONFIG_UBOOT
#include <asm/ppcboot.h>

extern unsigned char __res[];
bd_t *bd = (bd_t *)__res;

#define FLASH_ADDR  (bd->bi_flashstart)
#define FLASH_SIZE  (bd->bi_flashsize)
#else
#define FLASH_ADDR  CONFIG_PPC_5xxx_FLASH_ADDR
#define FLASH_SIZE  CONFIG_PPC_5xxx_FLASH_SIZE
#endif

#ifndef CONFIG_MTD_PARTITIONS
#error need to define CONFIG_MTD_PARTITIONS
#endif
static struct mtd_info *mymtd;

struct map_info uc101_map = {
	name:		"UC101",
	bankwidth:	2,
};

static struct mtd_partition uc101_partition[] =
{
	{
		name:		"Kernel",
                size:           0x00100000,
		offset:		0
	},
	{
		name:		"RootFS",
		size:		0x00200000,
		offset:		MTDPART_OFS_APPEND
	},
	{
		name:		"user",
		size:		0x00400000,
		offset:		MTDPART_OFS_APPEND
	},
	{
		name:		"U-boot",
		size:		0x00040000,
		offset:		MTDPART_OFS_APPEND
	},
	{
		name:		"Env",
		size:		0x00010000,
		offset:		MTDPART_OFS_APPEND
	},
	{
		name:		"red. Env",
		size:		0x00010000,
		offset:		MTDPART_OFS_APPEND
	},
	{
		name:		"reserve",
		size:		0x000a0000,
		offset:		MTDPART_OFS_APPEND
	},
};

#define NB_OF(x)	(sizeof(x)/sizeof(x[0]))

static int __init init_uc101 (void)
{
	printk (KERN_ALERT "UC101 flash device: %lx at %lx\n", FLASH_SIZE,
		FLASH_ADDR);
	uc101_map.virt = FLASH_ADDR;
	uc101_map.phys = FLASH_ADDR;
	uc101_map.size = FLASH_SIZE;
	simple_map_init(&uc101_map);

	mymtd = do_map_probe ("cfi_probe", &uc101_map);
	if (!mymtd) {
		printk ("cfi_probe failed\n");
		return -ENXIO;
	}

	mymtd->owner = THIS_MODULE;
	printk (KERN_NOTICE "Using static partition definition\n");
	add_mtd_partitions (mymtd, uc101_partition, NB_OF (uc101_partition));

	return 0;
}


static void __exit cleanup_uc101 (void)
{
	if (mymtd) {
		del_mtd_device (mymtd);
		map_destroy (mymtd);
	}
}
module_init(init_uc101);
module_exit(cleanup_uc101);

MODULE_LICENSE("GPL");
MODULE_AUTHOR("");
MODULE_DESCRIPTION("MTD map driver for UC101 board");
