/*
 * drivers/spi/spi_hmi1001.c
 *
 * SPI adapter driver for MPC5200 based board
 *
 * 2005 (c) Wolfgang Denk, DENX Software Engineering, <wd@denx.de>
 *
 * This file is licensed under the term of  the GNU General Public License
 * version 2. The program licensed "as is" without any warranty of any
 * kind, whether express or implied.
 */

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/init.h>
#include <asm/mpc5xxx.h>

#include <linux/spi.h>
#include "spi-algo-mpc5xxx-psc.h"

#define HMI1001_SPI_ADAPTER_NAME	"hmi1001"

	/* algorithm-specific data */
static struct spi_algo_mpc5xxx_psc_data hmi1001_algo_data = {
	(struct mpc5xxx_psc *)MPC5xxx_PSC3, /* use PSC3 */
};

	/* adapter-specific data */
static struct spi_adapter hmi1001_spi_adapter = {
	.name		= HMI1001_SPI_ADAPTER_NAME,
	.algo		= NULL,
	.algo_data	= &hmi1001_algo_data,  /* algorithm specific data */
};

	/* module initialization function */	
static int __init hmi1001_spi_init(void)
{
	printk("HMI1001 SPI adapter driver: Revision 0.1\n");
	return spi_mpc5xxx_psc_add_bus(&hmi1001_spi_adapter);
}

	/* module uninitialization function */
static void __exit hmi1001_spi_cleanup(void)
{
	int ret = spi_mpc5xxx_psc_del_bus(&hmi1001_spi_adapter);
	printk("HMI1001 SPI adapter driver unloaded (ret=%d)\n", ret);
}

module_init(hmi1001_spi_init);
module_exit(hmi1001_spi_cleanup);

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Wolfgang Denk, <wd@denx.de>");
MODULE_DESCRIPTION("HMI1001 SPI adapter");
